@@ -18,28 +18,26 @@
 #define dont_support_blurred
 
 
-#if CYGWIN_VERSION_API_MINOR >= 222
 static wstring rc_filename = 0;
-#else
-static string rc_filename = 0;
-#endif
 
 const config default_cfg = {
   // Looks
   .fg_colour = 0xBFBFBF,
   .bold_colour = (colour)-1,
   .bg_colour = 0x000000,
+  .cursor_colour = 0xBFBFBF,
   .search_fg_colour = 0x000000,
   .search_bg_colour = 0x00DDDD,
   .search_current_colour = 0x0099DD,
-  .cursor_colour = 0xBFBFBF,
+  .theme_file = L"",
   .transparency = 0,
   .blurred = false,
   .opaque_when_focused = false,
   .cursor_type = CUR_LINE,
   .cursor_blinks = true,
   // Text
-  .font = {.name = "Lucida Console", .isbold = false, .size = 9},
+  .font = {.name = L"Lucida Console", .size = 9, .weight = 400, .isbold = false},
+  .show_hidden_fonts = false,
   .font_smoothing = FS_DEFAULT,
   .bold_as_font = -1,  // -1 means "the opposite of bold_as_colour"
   .bold_as_colour = true,
@@ -134,8 +132,7 @@ const config default_cfg = {
   }
 };
 
-config cfg, new_cfg;
-static config file_cfg;
+config cfg, new_cfg, file_cfg;
 
 typedef enum {
   OPT_BOOL, OPT_MOD, OPT_TRANS, OPT_CURSOR, OPT_FONTSMOOTH,
@@ -152,14 +149,19 @@ static const struct {
   ushort offset;
 }
 options[] = {
-  // Looks
+  // Colour base options;
+  // check_legacy_options() assumes these are the first three here:
   {"ForegroundColour", OPT_COLOUR, offcfg(fg_colour)},
-  {"BoldColour", OPT_COLOUR, offcfg(bold_colour)},
   {"BackgroundColour", OPT_COLOUR, offcfg(bg_colour)},
+  {"UseSystemColours", OPT_BOOL | OPT_LEGACY, offcfg(use_system_colours)},
+
+  // Looks
+  {"BoldColour", OPT_COLOUR, offcfg(bold_colour)},
+  {"CursorColour", OPT_COLOUR, offcfg(cursor_colour)},
   {"SearchForegroundColour", OPT_COLOUR, offcfg(search_fg_colour)},
   {"SearchBackgroundColour", OPT_COLOUR, offcfg(search_bg_colour)},
   {"SearchCurrentColour", OPT_COLOUR, offcfg(search_current_colour)},
-  {"CursorColour", OPT_COLOUR, offcfg(cursor_colour)},
+  {"ThemeFile", OPT_WSTRING, offcfg(theme_file)},
   {"Transparency", OPT_TRANS, offcfg(transparency)},
 #ifdef support_blurred
   {"Blur", OPT_BOOL, offcfg(blurred)},
@@ -169,9 +171,12 @@ options[] = {
   {"CursorBlinks", OPT_BOOL, offcfg(cursor_blinks)},
 
   // Text
-  {"Font", OPT_STRING, offcfg(font.name)},
-  {"FontIsBold", OPT_BOOL, offcfg(font.isbold)},
+  {"Font", OPT_WSTRING, offcfg(font.name)},
+  {"FontSize", OPT_INT | OPT_LEGACY, offcfg(font.size)},
   {"FontHeight", OPT_INT, offcfg(font.size)},
+  {"FontWeight", OPT_INT, offcfg(font.weight)},
+  {"FontIsBold", OPT_BOOL, offcfg(font.isbold)},
+  {"ShowHiddenFonts", OPT_BOOL, offcfg(show_hidden_fonts)},
   {"FontSmoothing", OPT_FONTSMOOTH, offcfg(font_smoothing)},
   {"BoldAsFont", OPT_BOOL, offcfg(bold_as_font)},
   {"BoldAsColour", OPT_BOOL, offcfg(bold_as_colour)},
@@ -276,7 +281,6 @@ options[] = {
   {"BoldWhite", OPT_COLOUR, offcfg(ansi_colours[BOLD_WHITE_I])},
 
   // Legacy
-  {"UseSystemColours", OPT_BOOL | OPT_LEGACY, offcfg(use_system_colours)},
   {"BoldAsBright", OPT_BOOL | OPT_LEGACY, offcfg(bold_as_colour)},
   {"FontQuality", OPT_FONTSMOOTH | OPT_LEGACY, offcfg(font_smoothing)},
 };
@@ -362,6 +366,14 @@ static opt_val
   }
 };
 
+
+#ifdef debug_theme
+#define trace_theme(params)	printf params
+#else
+#define trace_theme(params)
+#endif
+
+
 static int
 find_option(string name)
 {
@@ -373,7 +385,8 @@ find_option(string name)
   return -1;
 }
 
-static uchar file_opts[lengthof(options)], arg_opts[lengthof(options)];
+static uchar file_opts[lengthof(options)];
+static uchar arg_opts[lengthof(options)];
 static uint file_opts_num, arg_opts_num;
 
 static bool
@@ -382,13 +395,6 @@ seen_file_option(uint i)
   return memchr(file_opts, i, file_opts_num);
 }
 
-static void
-remember_file_option(uint i)
-{
-  if (!seen_file_option(i))
-    file_opts[file_opts_num++] = i;
-}
-
 static bool
 seen_arg_option(uint i)
 {
@@ -396,8 +402,29 @@ seen_arg_option(uint i)
 }
 
 static void
-remember_arg_option(uint i)
+remember_file_option(char * tag, uint i)
 {
+  (void)tag;
+  trace_theme(("[%s] remember_file_option (file %d arg %d) %d %s\n", tag, seen_file_option(i), seen_arg_option(i), i, options[i].name));
+  if (i > 255) {
+    fprintf(stderr, "Internal error: too many options.\n");
+    exit(1);
+  }
+
+  if (!seen_file_option(i))
+    file_opts[file_opts_num++] = i;
+}
+
+static void
+remember_arg_option(char * tag, uint i)
+{
+  (void)tag;
+  trace_theme(("[%s] remember_arg_option (file %d arg %d) %d %s\n", tag, seen_file_option(i), seen_arg_option(i), i, options[i].name));
+  if (i > 255) {
+    fprintf(stderr, "Internal error: too many options.\n");
+    exit(1);
+  }
+
   if (!seen_arg_option(i))
     arg_opts[arg_opts_num++] = i;
 }
@@ -405,11 +432,11 @@ remember_arg_option(uint i)
 void
 remember_arg(string option)
 {
-  remember_arg_option(find_option(option));
+  remember_arg_option("rem_arg", find_option(option));
 }
 
 static void
-check_legacy_options(void (*remember_option)(uint))
+check_legacy_options(void (*remember_option)(char * tag, uint))
 {
   if (cfg.use_system_colours) {
     // Translate 'UseSystemColours' to colour settings.
@@ -419,9 +446,9 @@ check_legacy_options(void (*remember_option)(uint))
 
     // Make sure they're written to the config file.
     // This assumes that the colour options are the first three in options[].
-    remember_option(0);
-    remember_option(1);
-    remember_option(2);
+    remember_option("legacy", 0);
+    remember_option("legacy", 1);
+    remember_option("legacy", 2);
   }
 }
 
@@ -550,7 +577,7 @@ static void
 check_arg_option(int i)
 {
   if (i >= 0) {
-    remember_arg_option(i);
+    remember_arg_option("chk_arg", i);
     check_legacy_options(remember_arg_option);
   }
 }
@@ -567,9 +594,39 @@ parse_arg_option(string option)
   check_arg_option(parse_option(option, false));
 }
 
+void
+load_theme(wstring theme)
+{
+  wchar * theme_file = (wchar *)theme;
+  bool free_theme_file = false;
+  if (*theme && !wcschr(theme, L'/') && !wcschr(theme, L'\\')) {
+    string subfolder = ".mintty/themes";
+    char rcdir[strlen(home) + strlen(subfolder) + 2];
+    sprintf(rcdir, "%s/%s", home, subfolder);
+    wchar * rcpat = path_posix_to_win_w(rcdir);
+    int len = wcslen(rcpat);
+    rcpat = renewn(rcpat, len + wcslen(theme_file) + 2);
+    rcpat[len++] = L'/';
+    wcscpy(&rcpat[len], theme_file);
+    theme_file = rcpat;
+    free_theme_file = true;
+  }
+  char * filename = path_win_w_to_posix(theme_file);
+  if (free_theme_file)
+    free(theme_file);
+  load_config(filename, false);
+  free(filename);
+}
+
 void
 load_config(string filename, bool to_save)
 {
+  trace_theme(("load_config <%s> %d\n", filename, to_save));
+  if (!to_save) {
+    // restore base configuration, without theme mix-ins
+    copy_config("load", &cfg, &file_cfg);
+  }
+
   if (access(filename, R_OK) == 0 && access(filename, W_OK) < 0)
     to_save = false;
 
@@ -577,11 +634,7 @@ load_config(string filename, bool to_save)
     file_opts_num = arg_opts_num = 0;
 
     delete(rc_filename);
-#if CYGWIN_VERSION_API_MINOR >= 222
-    rc_filename = cygwin_create_path(CCP_POSIX_TO_WIN_W, filename);
-#else
-    rc_filename = strdup(filename);
-#endif
+    rc_filename = path_posix_to_win_w(filename);
   }
 #ifdef debug_config
   printf ("will save to %s? %d\n", filename, to_save);
@@ -595,7 +648,7 @@ load_config(string filename, bool to_save)
       if (line[0] != '#' && line[0] != '\0') {
         int i = parse_option(line, true);
         if (to_save && i >= 0)
-          remember_file_option(i);
+          remember_file_option("load", i);
       }
     }
     fclose(file);
@@ -603,12 +656,22 @@ load_config(string filename, bool to_save)
 
   check_legacy_options(remember_file_option);
 
-  copy_config(&file_cfg, &cfg);
+  if (to_save) {
+    copy_config("after load", &file_cfg, &cfg);
+  }
 }
 
 void
-copy_config(config *dst_p, const config *src_p)
+copy_config(char * tag, config * dst_p, const config * src_p)
 {
+#ifdef debug_theme
+  char * cfg(config * p) {
+    return p == new_cfg ? "new" : p == file_cfg ? "file" : p == cfg ? "cfg" : "?";
+  }
+  printf("[%s] copy_config %s <- %s\n", tag, cfg(dst_p), cfg(src_p));
+#else
+  (void)tag;
+#endif
   for (uint i = 0; i < lengthof(options); i++) {
     opt_type type = options[i].type;
     if (!(type & OPT_LEGACY)) {
@@ -632,7 +695,7 @@ copy_config(config *dst_p, const config *src_p)
 void
 init_config(void)
 {
-  copy_config(&cfg, &default_cfg);
+  copy_config("init", &cfg, &default_cfg);
 }
 
 void
@@ -650,7 +713,7 @@ finish_config(void)
   // bold_as_font used to be implied by !bold_as_colour.
   if (cfg.bold_as_font == -1) {
     cfg.bold_as_font = !cfg.bold_as_colour;
-    remember_file_option(find_option("BoldAsFont"));
+    remember_file_option("finish", find_option("BoldAsFont"));
   }
 
   if (0 < cfg.transparency && cfg.transparency <= 3)
@@ -662,11 +725,7 @@ save_config(void)
 {
   string filename;
 
-#if CYGWIN_VERSION_API_MINOR >= 222
-  filename = cygwin_create_path(CCP_WIN_W_TO_POSIX, rc_filename);
-#else
-  filename = rc_filename;
-#endif
+  filename = path_win_w_to_posix(rc_filename);
 
   FILE *file = fopen(filename, "w");
 
@@ -687,7 +746,8 @@ save_config(void)
       opt_type type = options[i].type;
       if (!(type & OPT_LEGACY)) {
         fprintf(file, "%s=", options[i].name);
-        void *cfg_p = seen_arg_option(i) ? &file_cfg : &cfg;
+        //?void *cfg_p = seen_arg_option(i) ? &file_cfg : &cfg;
+        void *cfg_p = &file_cfg;
         void *val_p = cfg_p + options[i].offset;
         switch (type) {
           when OPT_STRING:
@@ -722,22 +782,21 @@ save_config(void)
     fclose(file);
   }
 
-#if CYGWIN_VERSION_API_MINOR >= 222
   delete(filename);
-#endif
 }
 
 
 static control *cols_box, *rows_box, *locale_box, *charset_box;
 
-static void
+void
 apply_config(bool save)
 {
   // Record what's changed
   for (uint i = 0; i < lengthof(options); i++) {
     opt_type type = options[i].type;
     uint offset = options[i].offset;
-    void *val_p = (void *)&cfg + offset;
+    //void *val_p = (void *)&cfg + offset;
+    void *val_p = (void *)&file_cfg + offset;
     void *new_val_p = (void *)&new_cfg + offset;
     bool changed;
     switch (type) {
@@ -750,13 +809,23 @@ apply_config(bool save)
       otherwise:
         changed = (*(char *)val_p != *(char *)new_val_p);
     }
-    if (changed && !seen_arg_option(i))
-      remember_file_option(i);
+    if (changed) {
+      remember_file_option("apply", i);
+    }
   }
 
-  win_reconfig();
+  copy_config("apply", &file_cfg, &new_cfg);
+  win_reconfig();  // copy_config(&cfg, &new_cfg);
   if (save)
     save_config();
+  bool had_theme = !!*cfg.theme_file;
+
+  if (*cfg.theme_file) {
+    load_theme(cfg.theme_file);
+    win_reset_colours();
+  }
+  else if (had_theme)
+    win_reset_colours();
 }
 
 static void
@@ -801,7 +870,7 @@ current_size_handler(control *unused(ctrl), int event)
 }
 
 static void
-printerbox_handler(control *ctrl, int event)
+printer_handler(control *ctrl, int event)
 {
   const wstring NONE = L"◇ None (printing disabled) ◇";  // ♢◇
   const wstring CFG_NONE = L"";
@@ -964,10 +1033,96 @@ bell_handler(control *ctrl, int event)
         }
       }
       free(beep);
+      win_bell(&new_cfg);
     }
   }
 }
 
+#include "winpriv.h"  // home
+
+static void
+add_file_resources(control *ctrl, wstring pattern)
+{
+  string subfolder = ".mintty";
+  char rcdir[strlen(home) + strlen(subfolder) + 2];
+  sprintf(rcdir, "%s/%s", home, subfolder);
+  wchar * rcpat = path_posix_to_win_w(rcdir);
+  int len = wcslen(rcpat);
+  rcpat = renewn(rcpat, len + wcslen(pattern) + 2);
+  rcpat[len++] = L'/';
+  wcscpy(&rcpat[len], pattern);
+
+  wstring suf = wcsrchr(pattern, L'.');
+  int sufl = suf ? wcslen(suf) : 0;
+
+  WIN32_FIND_DATAW ffd;
+  HANDLE hFind = FindFirstFileW(rcpat, &ffd);
+  int ok = hFind != INVALID_HANDLE_VALUE;
+  //if (!ok) retry with "/usr/share/mintty"?
+  //(then check also win_bell() and load_theme())
+  while (ok) {
+    if (ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
+      // skip
+    }
+    else {
+      //LARGE_INTEGER filesize = {.LowPart = ffd.nFileSizeLow, .HighPart = ffd.nFileSizeHigh};
+      //long s = filesize.QuadPart;
+
+      // strip suffix
+      int len = wcslen(ffd.cFileName);
+      if (ffd.cFileName[0] != '.' && ffd.cFileName[len - 1] != '~') {
+        ffd.cFileName[len - sufl] = 0;
+        dlg_listbox_add_w(ctrl, ffd.cFileName);
+      }
+    }
+    ok = FindNextFileW(hFind, &ffd);
+  }
+  FindClose(hFind);
+}
+
+static void
+bellfile_handler(control *ctrl, int event)
+{
+  const wstring NONE = L"◇ None (system sound) ◇";  // ♢◇
+  const wstring CFG_NONE = L"";
+  wstring bell_file = new_cfg.bell_file;
+  if (event == EVENT_REFRESH) {
+    dlg_listbox_clear(ctrl);
+    dlg_listbox_add_w(ctrl, NONE);
+    add_file_resources(ctrl, L"sounds/*.wav"); //  dlg_listbox_add_w(ctrl, L"...");
+    // strip std dir prefix...
+    dlg_editbox_set_w(ctrl, *bell_file ? bell_file : NONE);
+  }
+  else if (event == EVENT_VALCHANGE || event == EVENT_SELCHANGE) {
+    dlg_editbox_get_w(ctrl, &bell_file);
+    if (!wcscmp(bell_file, NONE))
+      wstrset(&bell_file, CFG_NONE);
+    // add std dir prefix?
+    new_cfg.bell_file = bell_file;
+    win_bell(&new_cfg);
+  }
+}
+
+static void
+theme_handler(control *ctrl, int event)
+{
+  const wstring NONE = L"◇ None ◇";  // ♢◇
+  const wstring CFG_NONE = L"";
+  wstring theme_file = new_cfg.theme_file;
+  if (event == EVENT_REFRESH) {
+    dlg_listbox_clear(ctrl);
+    dlg_listbox_add_w(ctrl, NONE);
+    add_file_resources(ctrl, L"themes/*"); //  dlg_listbox_add_w(ctrl, L"...");
+    dlg_editbox_set_w(ctrl, *theme_file ? theme_file : NONE);
+  }
+  else if (event == EVENT_VALCHANGE || event == EVENT_SELCHANGE) {
+    dlg_editbox_get_w(ctrl, &theme_file);
+    if (!wcscmp(theme_file, NONE))
+      wstrset(&theme_file, CFG_NONE);
+    new_cfg.theme_file = theme_file;
+  }
+}
+
 static void
 bell_tester(control *unused(ctrl), int event)
 {
@@ -1012,6 +1167,9 @@ setup_config_box(controlbox * b)
   ctrl_pushbutton(
     s, "&Cursor...", dlg_stdcolour_handler, &new_cfg.cursor_colour
   )->column = 2;
+  ctrl_combobox(
+    s, "&Theme", 80, theme_handler, &new_cfg.theme_file
+  );
 
   s = ctrl_new_set(b, "Looks", "Transparency");
   bool with_glass = win_is_glass_available();
@@ -1252,7 +1410,7 @@ setup_config_box(controlbox * b)
     s, "&Answerback", 100, dlg_stdstringbox_handler, &new_cfg.answerback
   )->column = 1;
 
-  s = ctrl_new_set(b, "Terminal", "Bell sound (overridden by BellFreq or BellFile)");
+  s = ctrl_new_set(b, "Terminal", "Bell (sound overridden by Wave/BellFile or BellFreq)");
   ctrl_columns(s, 3, 36, 19, 45);
   ctrl_combobox(
     s, null, 100, bell_handler, 0
@@ -1264,18 +1422,15 @@ setup_config_box(controlbox * b)
     s, "&Highlight in taskbar", dlg_stdcheckbox_handler, &new_cfg.bell_taskbar
   )->column = 2;
   ctrl_columns(s, 1, 100);  // reset column stuff so we can rearrange them
-  ctrl_columns(s, 2, 80, 20);
-#ifdef use_bellfileselection
-  ctrl_combobox(
-    s, "&Wave", 80,
-    bellfile_selector, &new_cfg.bell_file
-  )->column = 0;
-#else
+  ctrl_columns(s, 2, 82, 18);
 #ifdef use_belleditbox
   ctrl_editbox(
-    s, "&Wave", 80, dlg_stdstringbox_handler, &new_cfg.bell_file
+    s, "&Wave", 83, dlg_stdstringbox_handler, &new_cfg.bell_file
+  )->column = 0;
+#else
+  ctrl_combobox(
+    s, "&Wave", 83, bellfile_handler, &new_cfg.bell_file
   )->column = 0;
-#endif
 #endif
   ctrl_pushbutton(
     s, "&Play", bell_tester, 0
@@ -1284,11 +1439,11 @@ setup_config_box(controlbox * b)
   s = ctrl_new_set(b, "Terminal", "Printer");
 #ifdef use_multi_listbox_for_printers
   ctrl_listbox(
-    s, null, 4, 100, printerbox_handler, 0
+    s, null, 4, 100, printer_handler, 0
   );
 #else
   ctrl_combobox(
-    s, null, 100, printerbox_handler, 0
+    s, null, 100, printer_handler, 0
   );
 #endif
 
