@@ -32,8 +32,9 @@ static inline uchar blue(colour c) { return c >> 16; }
 // Font properties.
 
 typedef struct {
-  string name;
+  wstring name;
   int size;
+  int weight;
   bool isbold;
 } font_spec;
 
@@ -44,6 +45,7 @@ typedef struct {
   // Looks
   colour fg_colour, bold_colour, bg_colour, cursor_colour;
   colour search_fg_colour, search_bg_colour, search_current_colour;
+  wstring theme_file;
   char transparency;
   bool blurred;
   bool opaque_when_focused;
@@ -51,6 +53,7 @@ typedef struct {
   bool cursor_blinks;
   // Text
   font_spec font;
+  bool show_hidden_fonts;
   char font_smoothing;
   char bold_as_font;    // 0 = false, 1 = true, -1 = undefined
   bool bold_as_colour;
@@ -129,14 +132,16 @@ typedef struct {
   bool use_system_colours;
 } config;
 
-extern config cfg, new_cfg;
+extern config cfg, new_cfg, file_cfg;
 
 extern void init_config(void);
 extern void load_config(string filename, bool to_save);
+extern void load_theme(wstring theme);
 extern void set_arg_option(string name, string val);
 extern void parse_arg_option(string);
 extern void remember_arg(string);
 extern void finish_config(void);
-extern void copy_config(config *dst, const config *src);
+extern void copy_config(char * tag, config * dst, const config * src);
+extern void apply_config(bool save);
 
 #endif
