@@ -33,6 +33,10 @@
 #endif
 
 
+char * home;
+char * cmd;
+bool icon_is_from_shortcut = false;
+
 HINSTANCE inst;
 HWND wnd;
 HIMC imc;
@@ -46,61 +50,6 @@ static bool invoked_with_appid = false;
 #endif
 
 
-#if CYGWIN_VERSION_API_MINOR < 74 || defined(TEST_WCS)
-// needed for MinGW MSYS
-
-# if CYGWIN_VERSION_DLL_MAJOR >= 1005
-# define need_wcschr
-# endif
-# define dont_need_wcsdup
-
-# ifdef TEST_WCS
-#define wcsdup _wcsdup
-#define wcschr _wcschr
-#define wcsncmp _wcsncmp
-# endif
-
-#define wcscpy(tgt, src) memcpy(tgt, src, (wcslen(src) + 1) * sizeof(wchar))
-
-# ifdef need_wcsdup
-static wchar *
-wcsdup(const wchar * s)
-{
-  wchar * dup = newn(wchar, wcslen(s) + 1);
-  wcscpy(dup, s);
-  return dup;
-}
-# endif
-
-# ifdef need_wcschr
-static wchar *
-wcschr(const wchar * s, wchar c)
-{
-  while (* s) {
-    if (* s == c)
-      return (wchar *)s;
-    s ++;
-  }
-  return 0;
-}
-# endif
-
-static int
-wcsncmp(const wchar * s1, const wchar * s2, int len)
-{
-  for (int i = 0; i < len; i++)
-    if (s1[i] < s2[i])
-      return -1;
-    else if (s1[i] > s2[i])
-      return 1;
-    else if (s1[i] == 0)
-      return 0;
-  return 0;
-}
-
-#endif
-
-
 static int extra_width, extra_height, norm_extra_width, norm_extra_height;
 
 // State
@@ -756,7 +705,23 @@ void
 win_bell(config * conf)
 {
   if (conf->bell_sound || conf->bell_type) {
-    if (*conf->bell_file && PlaySoundW(conf->bell_file, NULL, SND_ASYNC | SND_FILENAME)) {
+    wchar * bell_file = (wchar *)conf->bell_file;
+    bool free_bell_file = false;
+    if (*bell_file && !wcschr(bell_file, L'/') && !wcschr(bell_file, L'\\')) {
+      string subfolder = ".mintty/sounds";
+      char rcdir[strlen(home) + strlen(subfolder) + 2];
+      sprintf(rcdir, "%s/%s", home, subfolder);
+      wchar * rcpat = path_posix_to_win_w(rcdir);
+      int len = wcslen(rcpat);
+      rcpat = renewn(rcpat, len + wcslen(bell_file) + 6);
+      rcpat[len++] = L'/';
+      wcscpy(&rcpat[len], bell_file);
+      len = wcslen(rcpat);
+      wcscpy(&rcpat[len], L".wav");
+      bell_file = rcpat;
+      free_bell_file = true;
+    }
+    if (*bell_file && PlaySoundW(bell_file, NULL, SND_ASYNC | SND_FILENAME)) {
       // played
     }
     else if (conf->bell_freq)
@@ -771,7 +736,11 @@ win_bell(config * conf)
       MessageBeep((conf->bell_type - 1) * 16);
     } else if (conf->bell_type < 0)
       MessageBeep(0xFFFFFFFF);
+
+    if (free_bell_file)
+      free(bell_file);
   }
+
   if (conf->bell_taskbar && !term.has_focus)
     flash_taskbar(true);
 }
@@ -961,7 +930,7 @@ font_cs_reconfig(bool font_changed)
 {
   if (font_changed) {
     win_init_fonts(cfg.font.size);
-    trace_resize((" (win_reconfig -> win_adapt_term_size)\n"));
+    trace_resize((" (font_cs_reconfig -> win_adapt_term_size)\n"));
     win_adapt_term_size(true, false);
   }
   win_update_scrollbar();
@@ -987,8 +956,9 @@ win_reconfig(void)
   term_reconfig();
 
   bool font_changed =
-    strcmp(new_cfg.font.name, cfg.font.name) ||
+    wcscmp(new_cfg.font.name, cfg.font.name) ||
     new_cfg.font.size != cfg.font.size ||
+    new_cfg.font.weight != cfg.font.weight ||
     new_cfg.font.isbold != cfg.font.isbold ||
     new_cfg.bold_as_font != cfg.bold_as_font ||
     new_cfg.bold_as_colour != cfg.bold_as_colour ||
@@ -1004,7 +974,7 @@ win_reconfig(void)
     win_set_colour(CURSOR_COLOUR_I, new_cfg.cursor_colour);
 
   /* Copy the new config and refresh everything */
-  copy_config(&cfg, &new_cfg);
+  copy_config("win_reconfig", &cfg, &new_cfg);
 
   font_cs_reconfig(font_changed);
 }
@@ -1277,6 +1247,7 @@ static struct {
         term_set_focus(false, true);
       }
       update_transparency();
+      win_key_reset();
 
     when WM_SETFOCUS:
       trace_resize(("# WM_SETFOCUS VK_SHIFT %02X\n", GetKeyState(VK_SHIFT)));
@@ -1470,6 +1441,13 @@ show_msg(FILE *stream, string msg)
     MessageBox(0, msg, APPNAME, MB_OK);
 }
 
+static void
+show_msg_w(FILE *stream, wstring msg)
+{
+  if (fprintf(stream, "%ls", msg) < 0 || fputs("\n", stream) < 0 || fflush(stream) < 0)
+    MessageBoxW(0, msg, _W(APPNAME), MB_OK);
+}
+
 static no_return __attribute__((format(printf, 1, 2)))
 error(char *format, ...)
 {
@@ -1484,6 +1462,7 @@ error(char *format, ...)
   exit(1);
 }
 
+#ifdef old_warn
 static void __attribute__((format(printf, 1, 2)))
 warn(char *format, ...)
 {
@@ -1495,6 +1474,23 @@ warn(char *format, ...)
   msg = asform("%s: %s\n", main_argv[0], msg);
   show_msg(stderr, msg);
 }
+#endif
+
+static void
+warnw(wstring msg, wstring file, wstring err)
+{
+#if CYGWIN_VERSION_API_MINOR >= 201
+  wstring format = (err && *err) ? L"%s: %ls '%ls':\n%ls" : L"%s: %ls '%ls'";
+  wchar mess[wcslen(format) + strlen(main_argv[0]) + wcslen(msg) + wcslen(file) + (err ? wcslen(err) : 0)];
+  swprintf(mess, lengthof(mess), format, main_argv[0], msg, file, err);
+  show_msg_w(stderr, mess);
+#else
+  //MinGW
+  (void)msg; (void)file; (void)err; (void)show_msg_w;
+  string mess = "could not load icon file";
+  show_msg(stderr, mess);
+#endif
+}
 
 void
 report_pos()
@@ -1816,7 +1812,7 @@ main(int argc, char *argv[])
   invoked_with_appid = sui.dwFlags & STARTF_TITLEISAPPID;
   // shortcut or AppId would be found in sui.lpTitle
 # ifdef debuglog
-  fprintf(mtlog, "shortcut %d %s\n", invoked_from_shortcut, (char *)cygwin_create_path(CCP_WIN_W_TO_POSIX, sui.lpTitle));
+  fprintf(mtlog, "shortcut %d %ls\n", invoked_from_shortcut, sui.lpTitle);
 # endif
 #endif
 
@@ -1921,6 +1917,9 @@ main(int argc, char *argv[])
               longopt[1] == '-' ? longopt : shortopt);
     }
   }
+  copy_config("main after -o", &file_cfg, &cfg);
+  if (*cfg.theme_file)
+    load_theme(cfg.theme_file);
 
   finish_config();
 
@@ -2003,6 +2002,7 @@ main(int argc, char *argv[])
   HICON large_icon = 0, small_icon = 0;
   if (*cfg.icon) {
     //string icon_file = strdup(cfg.icon);
+    // could use path_win_w_to_posix(cfg.icon) to avoid the locale trick below
     string icon_file = cs__wcstoutf(cfg.icon);
     uint icon_index = 0;
     char *comma = strrchr(icon_file, ',');
@@ -2015,49 +2015,43 @@ main(int argc, char *argv[])
         icon_index = 0;
     }
     SetLastError(0);
-#if CYGWIN_VERSION_API_MINOR >= 181
-# if HAS_LOCALES
+#if HAS_LOCALES
     char * valid_locale = setlocale(LC_CTYPE, 0);
     if (valid_locale) {
       valid_locale = strdup(valid_locale);
       setlocale(LC_CTYPE, "C.UTF-8");
-#  if CYGWIN_VERSION_API_MINOR >= 222
+# if CYGWIN_VERSION_API_MINOR >= 222
       cygwin_internal(CW_INT_SETLOCALE);  // fix internal locale
-#  endif
-    }
 # endif
-    wchar *win_icon_file = cygwin_create_path(CCP_POSIX_TO_WIN_W, icon_file);
-# if HAS_LOCALES
+    }
+#endif
+    wchar *win_icon_file = path_posix_to_win_w(icon_file);
+#if HAS_LOCALES
     if (valid_locale) {
       setlocale(LC_CTYPE, valid_locale);
-#  if CYGWIN_VERSION_API_MINOR >= 222
+# if CYGWIN_VERSION_API_MINOR >= 222
       cygwin_internal(CW_INT_SETLOCALE);  // fix internal locale
-#  endif
+# endif
       free(valid_locale);
     }
-# endif
+#endif
     if (win_icon_file) {
       ExtractIconExW(win_icon_file, icon_index, &large_icon, &small_icon, 1);
       free(win_icon_file);
     }
-#else
-    char win_icon_file[MAX_PATH];
-    cygwin_conv_to_win32_path(icon_file, win_icon_file);
-    ExtractIconExA(win_icon_file, icon_index, &large_icon, &small_icon, 1);
-#endif
     if (!large_icon) {
       small_icon = 0;
-      uint error = GetLastError();
-      if (error) {
-        char msg[1024];
-        FormatMessage(
+      uint err = GetLastError();
+      if (err) {
+        wchar winmsg[1024];
+        FormatMessageW(
           FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_MAX_WIDTH_MASK,
-          0, error, 0, msg, sizeof msg, 0
+          0, err, 0, winmsg, lengthof(winmsg), 0
         );
-        warn("could not load icon from '%s': %s", cs__wcstombs(cfg.icon), msg);
+        warnw(L"could not load icon file", cfg.icon, winmsg);
       }
       else
-        warn("could not load icon from '%s'", cs__wcstombs(cfg.icon));
+        warnw(L"could not load icon file", cfg.icon, L"");
     }
     delete(icon_file);
   }
