@@ -198,6 +198,11 @@ row_padding(int i, int e)
 #define trace_font(params)	
 #endif
 
+#ifndef TCI_SRCLOCALE
+//old MinGW
+#define TCI_SRCLOCALE 0x1000
+#endif
+
 static UINT
 get_default_charset()
 {
@@ -234,9 +239,7 @@ adjust_font_weights()
   int fw_norm_1 = 1000;
   int fw_bold_1 = 1001;
   bool font_found = false;
-#ifdef check_ansi
   bool ansi_found = false;
-#endif
   int default_charset = get_default_charset();
   bool cs_found = default_charset == DEFAULT_CHARSET;
 
@@ -249,10 +252,8 @@ adjust_font_weights()
     trace_font(("%ls %ld it %d cs %d %s\n", lfp->lfFaceName, lfp->lfWeight, lfp->lfItalic, lfp->lfCharSet, (lfp->lfPitchAndFamily & 3) == FIXED_PITCH ? "fixed" : ""));
 
     font_found = true;
-#ifdef check_ansi
     if (lfp->lfCharSet == ANSI_CHARSET)
       ansi_found = true;
-#endif
     if (lfp->lfCharSet == default_charset || lfp->lfCharSet == DEFAULT_CHARSET)
       cs_found = true;
 
@@ -281,12 +282,9 @@ adjust_font_weights()
     trace_font(("//\n"));
     return;
   }
-#ifdef check_ansi
-  if (!ansi_found)
-    MessageBoxW(0, L"Font does not support ANSI character range", cfg.font.name, MB_ICONWARNING);
-#endif
-  if (!cs_found)
-    MessageBoxW(0, L"Font does not support system locale", cfg.font.name, MB_ICONWARNING);
+  if (!ansi_found && !cs_found) {
+    MessageBoxW(0, L"Font has limited support for character ranges", cfg.font.name, MB_ICONWARNING);
+  }
 
   // find available widths closest to selected widths
   if (abs(fw_norm - fw_norm_0) <= abs(fw_norm - fw_norm_1) && fw_norm_0 > 0)
